/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2002-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

/**
 * iptostr - print a dottedQuad from a 32bit int
 * @ip: < IP, in network byte order.
 * 
 * quick abstraction since inet_ntoa takes a slightly strange structure.
 * 
 * Returns: dotted Quad string
 */
__inline__ char *iptostr(uint32_t ip)
{
   struct in_addr in;
   in.s_addr = ip;
   return inet_ntoa(in);
}

/**
 * get_ip_for_name - 
 * @name: 
 * @ip: > ip in BigEn.
 * 
 * Returns: int
 */
int get_ip_for_name_libc(char *name, uint32_t *ip)
{
   struct hostent *he;

   he = gethostbyname(name);
   if( he == NULL ) return -1;
   *ip = (uint32_t)(*(uint32_t*)(he->h_addr_list[0]));
   return 0;
}

/**
 * get_name_for_ip_libc - 
 * @name: <> buffer to store name in
 * @nlen: < max len for name
 * @ip: < ip in Be32.
 * 
 * 
 * Returns: int
 */
int get_name_for_ip_libc(char *name, size_t nlen, uint32_t ip)
{
   struct hostent *he;

   he = gethostbyaddr((const char *)&ip, sizeof(uint32_t), AF_INET);
   if( he == NULL ) {
      strncpy(name, iptostr(ip), nlen);
      return 0;
   }
   if( he->h_name == NULL ) {
      strncpy(name, iptostr(ip), nlen);
      return 0;
   }
   strncpy(name, he->h_name, nlen);
   return 0;
}

/* Need some of these to behave in more concrete maner across platforms
 * and stuff.
 */
int gulm_atoi(char *c)
{
  int x = 0;

  while ('0' <= *c && *c <= '9')
  {
    x = x * 10 + (*c - '0');
    c++;
  }

  return(x);
}

/**
 * gulm_aton - 
 * @ascii: < string of a dotted quad IP address
 * @ip: > the ip in cpu byte order.
 * 
 * 
 * Returns: int
 */
int gulm_aton(char *ascii, uint32_t *ip)
{
  uint32_t value;
  int x;

  *ip = 0;

  for (x = 0; x < 4; x++)
  {
    value = gulm_atoi(ascii);
    if (value > 255)
      return(-1);

    *ip = (*ip << 8) | value;

    if (x != 3)
    {
      for(; *ascii != '.' && *ascii != '\0'; ascii++) {
         if( (*ascii < '0' || *ascii > '9') ) {
            /* not a number. stop */
            return -1;
         }
      }
      if( *ascii == '\0' ) return(-1);

      ascii++;
    }
  }

  return(0);
}


/* vim: set ai cin et sw=3 ts=3 : */
